# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#

from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist

from wagtail import wagtaildocs

from markdown.util import etree


class Linker(object):
    def run(self, name, optstr):
        try:
            text = name
            if len(optstr):
                text = optstr[0]

            doc = wagtaildocs.models.Document.objects.get(title=name)
            url = doc.url
            a = etree.Element('a')
            a.set('href', url)
            a.text = text
            return a
        except ObjectDoesNotExist:
            return '[document "{}" not found]'.format(name)
        except MultipleObjectsReturned:
            return '[multiple documents "{}" found]'.format(name)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist

from wagtail.wagtailimages import get_image_model

from markdown.util import etree


# TODO: Default spec and class should be configurable, because they're
# dependent on how the project is set up.  Hard-coding of 'left',
# 'right' and 'full-width' should be removed.


class Linker(object):
    def run(self, fname, optstr):
        opts = {}

        opts['spec'] = 'width-500'
        opts['classname'] = 'left'

        for opt in optstr:
            bits = opt.split('=', 1)
            opt = bits[0]
            value = ''

            if len(bits) > 1:
                value = bits[1]

            if opt == 'left':
                opts['classname'] = 'left'
            elif opt == 'right':
                opts['classname'] = 'right'
            elif opt == 'full':
                opts['classname'] = 'full-width'
            elif opt == 'width':
                try:
                    opts['spec'] = "width-%d" % int(value)
                except ValueError:
                    pass
        try:
            image = get_image_model().objects.get(title=fname)
        except ObjectDoesNotExist:
            return '[image "{}" not found]'.format(fname)
        except MultipleObjectsReturned:
            return '[multiple images "{}" found]'.format(fname)

        image_url = image.file.url
        rendition = image.get_rendition(opts['spec'])

        a = etree.Element('a')
        a.set('data-toggle', 'lightbox')
        a.set('data-type', 'image')
        a.set('href', image_url)
        img = etree.SubElement(a, 'img')
        img.set('src', rendition.url)
        img.set('class', opts['classname'])
        img.set('width', str(rendition.width))
        img.set('height', str(rendition.height))
        return a



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# tomasz.knapik@torchbox.com 2017-12-07
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
from django import forms

from wagtail.utils.widgets import WidgetWithScript


class MarkdownTextarea(WidgetWithScript, forms.widgets.Textarea):
    def __init__(self, **kwargs):
        super(MarkdownTextarea, self).__init__(**kwargs)

    def render_js_init(self, id_, name, value):
        return 'simplemdeAttach("{0}");'.format(id_)

    @property
    def media(self):
        return forms.Media(
            css={
                'all': (
                    'wagtailmarkdown/css/simplemde.min.css',
                )
            },
            js=(
                'wagtailmarkdown/js/simplemde.min.js',
                'wagtailmarkdown/js/simplemde.attach.js',
            )
        )



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
import warnings

from django.db.models import TextField

from .blocks import MarkdownBlock
from .warnings import WagtailMarkdownDeprecationWarning
from .widgets import MarkdownTextarea


class MarkdownBlock(MarkdownBlock):
    def __init__(self, *args, **kwargs):
        super(MarkdownBlock, self).__init__(*args, **kwargs)
        warning = (
            "Importing `wagtailmarkdown.fields.MarkdownBlock` is depreceated. "
            "Please use the `wagtailmarkdown.blocks.MarkdownBlock` import "
            "instead."
        )
        warnings.warn(warning, WagtailMarkdownDeprecationWarning, stacklevel=2)


class MarkdownField(TextField):
    def formfield(self, **kwargs):
        defaults = {
            'widget': MarkdownTextarea
        }
        defaults.update(kwargs)
        return super(MarkdownField, self).formfield(**defaults)

    def __init__(self, **kwargs):
        super(MarkdownField, self).__init__(**kwargs)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#
import warnings

from django.utils.safestring import mark_safe

import markdown
import bleach

from .mdx import tables
from .mdx import linker
from .warnings import WagtailMarkdownDeprecationWarning


def render_markdown(text, context=None):
    """
    Turn markdown into HTML.
    """
    if context is None or not isinstance(context, dict):
        context = {}
    markdown_html = _transform_markdown_into_html(text)
    sanitised_markdown_html = _sanitise_markdown_html(markdown_html)
    return mark_safe(sanitised_markdown_html)


def _transform_markdown_into_html(text):
    return markdown.markdown(text, **_get_markdown_kwargs())


def _sanitise_markdown_html(markdown_html):
    return bleach.clean(markdown_html, **_get_bleach_kwargs())


def _get_bleach_kwargs():
    bleach_kwargs = {}
    bleach_kwargs['tags'] = [
        'p',
        'div',
        'span',
        'h1',
        'h2',
        'h3',
        'h4',
        'h5',
        'h6',
        'tt',
        'pre',
        'em',
        'strong',
        'ul',
        'li',
        'dl',
        'dd',
        'dt',
        'code',
        'img',
        'a',
        'table',
        'tr',
        'th',
        'td',
        'tbody',
        'caption',
        'colgroup',
        'thead',
        'tfoot',
        'blockquote',
        'ol',
        '<hr>',
    ]
    bleach_kwargs['attributes'] = {
        '*': [
            'class',
            'style',
            'id',
        ],
        'a': [
            'href',
            'target',
            'rel',
        ],
        'img': [
            'src',
            'alt',
        ],
        'tr': [
            'rowspan',
            'colspan',
        ],
        'td': [
            'rowspan',
            'colspan',
            'align',
        ],
    }
    bleach_kwargs['styles'] = [
        'color',
        'background-color',
        'font-family',
        'font-weight',
        'font-size',
        'width',
        'height',
        'text-align',
        'border',
        'border-top',
        'border-bottom',
        'border-left',
        'border-right',
        'padding',
        'padding-top',
        'padding-bottom',
        'padding-left',
        'padding-right',
        'margin',
        'margin-top',
        'margin-bottom',
        'margin-left',
        'margin-right',
    ]
    return bleach_kwargs


def _get_markdown_kwargs():
    markdown_kwargs = {}
    markdown_kwargs['extensions'] = [
        'extra',
        'codehilite',
        tables.TableExtension(),
        linker.LinkerExtension({
             '__default__': 'wagtailmarkdown.mdx.linkers.page',
             'page:': 'wagtailmarkdown.mdx.linkers.page',
             'image:': 'wagtailmarkdown.mdx.linkers.image',
             'doc:': 'wagtailmarkdown.mdx.linkers.document',
         })
    ]
    markdown_kwargs['extension_configs'] = {
        'codehilite': [
            ('guess_lang', False),
        ]
    }
    markdown_kwargs['output_format'] = 'html5'
    return markdown_kwargs


def render(text, context=None):
    """
    Depreceated call to render_markdown().
    """
    warning = (
        "wagtailmarkdown.utils.render() is deprecated. Use "
        "wagtailmarkdown.utils.render_markdown() instead."
    )
    warnings.warn(warning, WagtailMarkdownDeprecationWarning, stacklevel=2)
    return render_markdown(text, context)



#! /usr/bin/env/python
# vim:sw=4 ts=4 et:
#
# Copyright (c) 2015 Torchbox Ltd.
# felicity@torchbox.com 2015-09-14
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#

from setuptools import setup, find_packages


README = 'https://github.com/torchbox/wagtail-markdown/blob/master/README.md'


setup(
    name='wagtail-markdown',
    version='0.5a1',
    description='Markdown support for Wagtail',
    long_description="Provides Markdown page field and streamfield block for "
                     "Wagtail. More info: {}".format(README),
    author='Felicity Tarnell',
    author_email='felicity@torchbox.com',
    url='https://github.com/torchbox/wagtail-markdown',
    install_requires=[
        'Markdown>=2.6,<2.7',
        'bleach>=1.4.2,<2.2',
    ],
    license='zlib',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'wagtailmarkdown': [
            'static/wagtailmarkdown/*/*'
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'License :: OSI Approved :: zlib/libpng License',
        'Programming Language :: Python :: 3',
    ],
)



# vim:sw=4 ts=4 et:
# Copyright (c) 2015 Torchbox Ltd.
# tomasz.knapik@torchbox.com 2017-12-07
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely. This software is provided 'as-is', without any express or implied
# warranty.
#


def setup():
    # Make warnings visible
    import warnings
    from .warnings import WagtailMarkdownDeprecationWarning
    warnings.simplefilter('default', WagtailMarkdownDeprecationWarning)


setup()


# Classes below are here so users who use old imports get
# meaningful errors.
class DepreceatedObject(object):
    def __init__(self, *args, **kwargs):
        warning = (
            "The `wagtailmarkdown.{class_name}` import is not valid. Please "
            "use `wagtailmarkdown.{new_path}` instead."
        ).format(
            new_path=self.new_path,
            class_name=self.class_name
        )
        raise ImportError(warning)


old_classes = [
    ('MarkdownBlock', 'blocks.MarkdownBlock'),
    ('MarkdownField', 'fields.MarkdownField'),
    ('MarkdownPanel', 'edit_handlers.MarkdownPanel'),
    ('MarkdownTextarea', 'widgets.MarkdownTextarea'),
]

for class_name, new_path in old_classes:
    locals()[class_name] = type(class_name, (DepreceatedObject,), {
        'class_name': class_name,
        'new_path': new_path,
    })
